@@ -17,3 +17,7 @@ setGeneric("eval", signature=c("expr", "envir"),
     }
 )
 
+evalq <- function(expr, envir = parent.frame(),
+                  enclos = if (is.list(envir) || is.pairlist(envir))
+                  parent.frame() else baseenv())
+  eval(substitute(expr), envir, enclos)
